@@ -1,60 +1,58 @@
 """Audio effects blocks."""
+from typing import Tuple, Callable
+import functools
 import math
 
 import numpy as np
 import scipy.signal
 import samplerate
 
+from klang.audio.filters import BackwardCombFilter
 from klang.audio.helpers import NYQUIST_FREQUENCY, get_silence
-from klang.audio.oscillators import Oscillator
-from klang.audio.waves import sine
+from klang.audio.oscillators import Oscillator, PwmOscillator
+from klang.audio.waves import square
 from klang.audio.wavfile import convert_samples_to_float, convert_samples_to_int
 from klang.block import Block
 from klang.composite import Composite
 from klang.config import BUFFER_SIZE, SAMPLING_RATE, KAMMERTON
 from klang.connections import Input, Relay
-from klang.constants import TAU, MONO, STEREO
-from klang.math import clip, blend
-from klang.music.tempo import compute_duration
+from klang.constants import PI, TAU, INF, MONO, STEREO
+from klang.math import clip, blend, linear_mapping
+from klang.music.tempo import compute_duration, TimeOrNoteValue
+from klang.primes import find_next_primes
 from klang.ring_buffer import RingBuffer
 
 
 __all__ = [
     'Gain', 'Tremolo', 'Delay', 'AudioSplitter', 'AudioCombiner', 'StereoDelay',
     'Filter', 'Subsampler', 'Bitcrusher', 'OctaveDistortion', 'TanhDistortion',
-    'PitchShifter', 'Transformer',
+    'PitchShifter', 'Transformer', 'Reverb',
 ]
 
 
-def sub_sample(array, skip):
-    """Sub / downsample sample array.
+@functools.lru_cache()
+def low_pass_coefficients(frequency: float) -> Tuple[list, list]:
+    """Filter coefficients for single pole low pass IIR filter. For decay use
+    frequency = 1. / decay. 0 and INF are supported as well.
 
-    Usage:
-        >>> samples = np.arange(6)
-        ... sub_sample(samples, 2)
-        array([0, 0, 2, 2, 4, 4])
+    Args:
+        frequency: Unnormalized cutoff frequency.
+
+    Returns:
+        Numerator / denominator coefficients.
+
+    Resources:
+        - https://tomroelandts.com/articles/low-pass-single-pole-iir-filter
     """
-    return np.repeat(array[..., ::skip], skip, axis=-1)
+    if frequency == 0:
+        return [0.], [1., -1.]
 
+    if frequency == INF:
+        return [1.], [1., 0.]
 
-def bit_crush(samples, nBits):
-    """Bit crush samples.
-
-    Usage:
-        >>> bit_crush(np.arange(-4, 4), nBits=1)
-        array([-4, -4, -2, -2,  0,  0,  2,  2])
-    """
-    return (samples >> nBits) << nBits
-
-
-def octave_distortion(samples):
-    """Non-linear octaver distortion."""
-    return 2. * samples ** 2 - 1.
-
-
-def tanh_distortion(samples, drive=1.):
-    """Tanh distortion. Only odd harmonics."""
-    return np.tanh(drive * samples)
+    a1 = -math.exp(-TAU * frequency / SAMPLING_RATE)
+    b0 = 1. - abs(a1)
+    return [b0], [1., a1]
 
 
 class Gain(Block):
@@ -62,49 +60,77 @@ class Gain(Block):
     """Simple gain block."""
 
     def __init__(self, gain=1.):
-        super().__init__(nInputs=1, nOutputs=1)
-        self.gain = gain
+        super().__init__(nInputs=2, nOutputs=1)
+        _, self.gain = self.inputs
+        self.gain.set_value(gain)
 
     def update(self):
+        gain = self.gain.value
         samples = self.input.value
-        self.output.set_value(self.gain * samples)
+        self.output.set_value(gain * samples)
 
 
 class Tremolo(Composite):
 
-    """LFO controlled amplitude modulation (AM)."""
+    """LFO controlled amplitude modulation.
 
-    def __init__(self, rate=5., intensity=1., wave_func=sine):
+    Support for different active phases (dutyCycle) and curve shape
+    (smoothness).
+
+    Attributes:
+        rate: Frequency relays to lfo.
+        depth: Value input for tremolo intensity.
+        smoothness: Value input for envelope shape.
+        dutyCycle: dutyCycle relays to lfo.
+        lfo: Pulse width modulation lfo.
+        zi: Low pass filter state.
+    """
+
+    def __init__(self, rate: TimeOrNoteValue = 3., depth: float = .8,
+                 smoothness: float = 1., dutyCycle: float = .5):
         """Kwargs:
-            rate (float): Initial effect frequency.
-            intensity (float): Initial effect intensity.
-            wave_func (function): Wave form function.
+            rate: Rate of tremolo.
+            depth: Intensity of amplitude modulation.
+            smoothness: Curve shape. 0. -> sine-like, 1. -> square.
+            dutyCycle: Active phase duration.
         """
         super().__init__(nOutputs=1)
-        _, self.rate, self.intensity = self.inputs = [
+        self.inputs = [
             Input(owner=self),
             Relay(owner=self),
             Input(owner=self),
+            Input(owner=self),
+            Relay(owner=self),
         ]
-        self.rate.set_value(rate),
-        self.intensity.set_value(intensity)
-        self.lfo = Oscillator(
-            frequency=rate,
-            wave_func=wave_func,
-            startPhase=TAU / 4., # Start from zero
-        )
+
+        _, self.rate, self.depth, self.smoothness, self.dutyCycle = self.inputs
+        self.rate.set_value(rate)
+        self.depth.set_value(depth)
+        self.smoothness.set_value(smoothness)
+        self.dutyCycle.set_value(dutyCycle)
+
+        self.lfo = PwmOscillator(rate)
         self.rate.connect(self.lfo.frequency)
+        self.dutyCycle.connect(self.lfo.dutyCycle)
+        self.zi = np.zeros(1)
 
     def update(self):
+        # Fetch input values
+        depth = self.depth.value
+        smoothness = self.smoothness.value
+        dutyCycle = self.dutyCycle.value
+
+        # Calculate AM-envelope
+        decay = max(1e-6, smoothness * min(dutyCycle, 1. - dutyCycle))
+        coeffs = low_pass_coefficients(1. / decay)
         self.lfo.update()
+        pwm = self.lfo.output.value
+        filteredPwm, self.zi = scipy.signal.lfilter(*coeffs, pwm, zi=self.zi)
+        env = (1. - depth) + depth * (.5 + .5 * filteredPwm)
 
-        # Calculate tremolo envelope in [0., 1.].
-        intensity = self.intensity.get_value()
-        mod = self.lfo.output.get_value()
-        env = 1. - clip(intensity, 0., 1.) * mod
-
-        samples = self.input.get_value()
-        self.output.set_value(env * samples)
+        # Apply
+        x = self.input.value
+        self.output.set_value(env * x)
 
 
 class Delay(Block):
@@ -125,10 +151,11 @@ class Delay(Block):
         self.feedback = feedback
         self.drywet = drywet
 
-        maxlen = int(self.MAX_TIME * SAMPLING_RATE)
         time = compute_duration(time)
-        delayOffset = int(time * SAMPLING_RATE)
-        self.buffer = RingBuffer.from_shape(maxlen, offset=delayOffset)
+        length = int(time * SAMPLING_RATE)
+        capacity = int(self.MAX_TIME * SAMPLING_RATE)
+
+        self.ring = RingBuffer(length, capacity)
 
     def validate_delay_time(self, time):
         if time > self.MAX_TIME:
@@ -142,8 +169,8 @@ class Delay(Block):
             nChannels, _ = new.shape
             new = new.sum(axis=0) / nChannels
 
-        old = self.buffer.read(BUFFER_SIZE).T
-        self.buffer.write((new + self.feedback * old).T)
+        old = self.ring.peek()
+        self.ring.extend(new + self.feedback * old)
         self.output.set_value(blend(new, old, self.drywet))
 
 
@@ -316,7 +343,7 @@ class Filter(Block):
 
     """Butterworth filter block.
 
-    TODO:
+    Todo:
       - Possible to use lfilter for multi channel audio?
     """
 
@@ -373,9 +400,20 @@ class Subsampler(Block):
         super().__init__(nInputs=1, nOutputs=1)
         self.factor = factor
 
+    @staticmethod
+    def sub_sample(array, skip):
+        """Sub / downsample sample array.
+
+        Usage:
+            >>> samples = np.arange(6)
+            ... sub_sample(samples, 2)
+            array([0, 0, 2, 2, 4, 4])
+        """
+        return np.repeat(array[..., ::skip], skip, axis=-1)
+
     def update(self):
         samples = self.input.get_value()
-        subSamples = sub_sample(samples, self.factor)
+        subSamples = self.sub_sample(samples, self.factor)
         self.output.set_value(subSamples)
 
 
@@ -394,12 +432,21 @@ class Bitcrusher(Block):
         super().__init__(nInputs=1, nOutputs=1)
         self.nBits = nBits
 
+    @staticmethod
+    def bit_crush(samples, nBits):
+        """Bit crush float samples.
+
+        Usage:
+            >>> bit_crush(np.arange(-4, 4), nBits=1)
+            array([-4, -4, -2, -2,  0,  0,  2,  2])
+        """
+        samplesI = convert_samples_to_int(samples)
+        crushedSamplesI = (samplesI >> nBits) << nBits
+        return convert_samples_to_float(crushedSamplesI)
+
     def update(self):
         samples = self.input.get_value()
-        samples = convert_samples_to_int(samples)
-        samples = bit_crush(samples, self.nBits)
-        samples = convert_samples_to_float(samples)
-        self.output.set_value(samples)
+        self.output.set_value(self.bit_crush(samples, self.nBits))
 
 
 class OctaveDistortion(Block):
@@ -409,9 +456,14 @@ class OctaveDistortion(Block):
     def __init__(self):
         super().__init__(nInputs=1, nOutputs=1)
 
+    @staticmethod
+    def octave_distortion(samples):
+        """Non-linear octaver distortion."""
+        return 2. * samples ** 2 - 1.
+
     def update(self):
         samples = self.input.get_value()
-        self.output.set_value(octave_distortion(samples))
+        self.output.set_value(self.octave_distortion(samples))
 
 
 class TanhDistortion(Block):
@@ -425,9 +477,14 @@ class TanhDistortion(Block):
         super().__init__(nInputs=1, nOutputs=1)
         self.drive = drive
 
+    @staticmethod
+    def tanh_distortion(samples, drive=1.):
+        """Tanh distortion. Only odd harmonics."""
+        return np.tanh(drive * samples)
+
     def update(self):
         samples = self.input.get_value()
-        self.output.set_value(tanh_distortion(samples, self.drive))
+        self.output.set_value(self.tanh_distortion(samples, self.drive))
 
 
 class PitchShifter(Block):
@@ -493,7 +550,112 @@ class Transformer(Block):
         width = upper - lower
         return cls(scale=.5 * width, offset=lower + .5 * width)
 
+    @classmethod
+    def from_ranges(cls, xRange=(0., 1.), yRange=(0., 1.)):
+        scale, offset = linear_mapping(xRange, yRange)
+        return cls(scale, offset)
+
+    def update(self):
+        self.output.set_value(
+            self.scale * self.input.value + self.offset
+        )
+
+
+class Reverb(Block):
+
+    """Simple comb filter / echo based reverb effect.
+
+    Prime number delay taps.
+
+    TODO:
+      - Different gain policies.
+      - Bandpass in feedback path of filters
+    """
+
+    def __init__(self, decay: float = 1.5, preDelay: float = .03, dryWet: float
+                 = .7, nEchos: int = 10, echoType: type = BackwardCombFilter):
+        """Kwargs:
+            decay: Decay value in seconds (~approx).
+            preDelay: Lower bound of all delay times.
+            dryWet: Amount of dry and effected audio portion.
+            nEchos: Number of echo taps.
+            echoType: Filter type for delay taps.
+        """
+        assert nEchos > 0
+        super().__init__(nInputs=1, nOutputs=1)
+        self.dryWet = clip(dryWet, 0., 1.)
+        self.filters = [
+            echoType(k, self.compute_alpha(k / SAMPLING_RATE, decay))
+            for k in find_next_primes(nEchos, int(preDelay * SAMPLING_RATE))
+        ]
+
+    @staticmethod
+    def compute_alpha(delayTime: float, decay: float) -> float:
+        """Compute gain value alpha for comb and echo filters.
+
+        Args:
+            delayTime: Delay time in seconds.
+            decay: Decay rate in seconds.
+
+        Returns:
+            alpha gain value.
+        """
+        if decay <= 0:
+            return 0.
+
+        return math.exp(-PI * delayTime / decay)
+
     def update(self):
         x = self.input.value
-        y = self.scale * x + self.offset
+        y = self.filters[0].filter(x)
+        for fil in self.filters[1:]:
+            y += fil.filter(x)
+
+        y /= len(self.filters)
+        self.output.set_value(blend(x, y, self.dryWet))
+
+
+class RingModulator(Composite):
+
+    """LFO modulated ring modulator."""
+
+    def __init__(self, frequency: float = 500., rate: float = 2., amount: float =
+                 20, wave_func: Callable = square, dryWet: float = 1.):
+        """Kwargs:
+            frequency: AM base frequency.
+            rate: LFO rate.
+            amount: Amount of FM modulation.
+            wave_func: LFO wave function.
+            dryWet: Blend level.
+        """
+        super().__init__()
+        self.inputs = [
+            Input(owner=self), Input(owner=self), Relay(owner=self),
+            Input(owner=self), Input(owner=self),
+        ]
+        self.outputs = [Relay(owner=self)]
+        _, self.frequency, self.rate, self.amount, self.dryWet = self.inputs
+        self.frequency.set_value(frequency)
+        self.rate.set_value(rate)
+        self.amount.set_value(amount)
+        self.dryWet.set_value(dryWet)
+
+        self.lfo = Oscillator(wave_func=wave_func)
+        self.modulator = Oscillator()
+
+        self.rate | self.lfo
+
+    def update(self):
+        # Frequency with modulation from lfo
+        self.lfo.update()
+        freqs = self.frequency.value + self.amount.value * self.lfo.output.value
+
+        # AM envelope from modulator
+        self.modulator.frequency.set_value(freqs)
+        self.modulator.update()
+        amSignal = self.modulator.output.value
+
+        # Blend signals
+        x = self.input.value
+        y = blend(x, amSignal * x, self.dryWet.value)
         self.output.set_value(y)
