@@ -1,7 +1,13 @@
 """Envelope generator blocks."""
 from klang.config import BUFFER_SIZE
 from klang.audio.helpers import DT
-from klang.audio._envelope import Envelope as _CEnvelope
+try:
+    # C Envelope
+    from klang.audio._envelope import Envelope
+except ImportError:
+    # Pure Python fallback
+    from klang.audio.envelope import Envelope
+from klang.audio.envelope import DEFAULT_OVERSHOOT
 from klang.block import Block
 from klang.connections import MessageInput
 
@@ -9,12 +15,12 @@ from klang.connections import MessageInput
 __all__ = ['ADSR', 'AR', 'D', 'R']
 
 
-class EnvelopeBase(Block, _CEnvelope):
-    def __init__(self, attack, decay, sustain, release, overshoot=1e-3,
-                 retrigger=False, loop=False):
+class EnvelopeBase(Block, Envelope):
+    def __init__(self, attack, decay, sustain, release,
+                 overshoot=DEFAULT_OVERSHOOT, retrigger=False, loop=False):
         super().__init__(nOutputs=1)
         self.inputs = self.trigger, = [MessageInput(owner=self)]
-        _CEnvelope.__init__(
+        Envelope.__init__(
             self,
             attack=attack,
             decay=decay,
