@@ -1,4 +1,4 @@
-from config import BUFFER_SIZE
+from klang.config import BUFFER_SIZE
 from klang.audio.helpers import get_silence, get_time
 from klang.audio.oscillators import Phasor
 from klang.block import Block
