@@ -1,11 +1,12 @@
 """Block execution."""
+from typing import List, Iterable, Tuple, Set
 import collections
 
-from klang.block import input_neighbors, output_neighbors
+from klang.block import input_neighbors, output_neighbors, Block
 from klang.graph import graph_matrix, topological_sorting
 
 
-def traverse_network(blocks):
+def traverse_network(blocks: Iterable[Block]) -> Tuple[List[Block], Set[Tuple[Block, Block]]]:
     """Traverse block network and collect vertices and edges. Vertices are
     returned as a list, not as a set, for preserving sequence order / better
     edge case handling.
@@ -34,7 +35,7 @@ def traverse_network(blocks):
     return vertices, edges
 
 
-def determine_execution_order(blocks):
+def determine_execution_order(blocks: Iterable[Block]) -> List[Block]:
     """Get appropriate execution order for block network."""
     blocks = list(blocks)
     vertices, edges = traverse_network(blocks)
@@ -58,13 +59,13 @@ def determine_execution_order(blocks):
     return execOrder
 
 
-def execute(blocks):
+def execute(blocks: Iterable[Block]):
     """Execute blocks."""
     for block in blocks:
         block.update()
 
 
-def print_exec_order(execOrder):
+def print_exec_order(execOrder: Iterable[Block]):
     """Print numbered execution order."""
     for iblock in enumerate(execOrder, 1):
         print('%d) %s' % iblock)
