@@ -22,12 +22,25 @@ def normalize_values(array):
 
 
 def clip(value, lower, upper):
-    """Clip value to [lower, upper]. Aka. clamp."""
+    """Clip value to [lower, upper]. Aka. clamp.
+
+    Usage:
+        >>> clip(-3, lower=-1, upper=2)
+        -1
+    """
     return min(max(value, lower), upper)
 
 
 def is_power_of_two(number):
-    """Check if number is power of two."""
+    """Check if number is power of two.
+
+    Usage:
+        >>> is_power_of_two(64)
+        True
+
+        >>> is_power_of_two(-3)
+        False
+    """
     if number <= 0:
         return False
 
@@ -60,3 +73,26 @@ def blend(a, b, x):
         array([0.5, 0.5, 0.5, 0.5])
     """
     return (1. - x) * a + x * b
+
+
+def sign(number):
+    """Math sign function."""
+    return math.copysign(1., number)
+
+
+def linear_mapping(xRange, yRange):
+    """Get linear coefficients for y = a * x + b.
+
+    Args:
+        xRange (tuple): Input range (xmin, xmax).
+        yRange (tuple): Output range (xmin, xmax).
+
+    Returns:
+        tuple: Linear coefficients a, b.
+    """
+    xmin, xmax = xRange
+    ymin, ymax = yRange
+    return np.linalg.solve([
+        [xmin, 1.],
+        [xmax, 1.],
+    ], [ymin, ymax])
